from django import forms
from .models import Article
from quillsafe.fields import QuillFormField

class ArticleForm(forms.ModelForm):
    content = QuillFormField()

    class Meta:
        model = Article
        fields = ["title", "content"]



# Generated by Django 5.2.6 on 2025-09-13 09:03

import quillsafe.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Article",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255)),
                ("content", quillsafe.fields.QuillField()),
            ],
        ),
    ]



import uuid
import json
from django import template
from django.utils.safestring import mark_safe

register = template.Library()

@register.simple_tag
def quill_display(content, height="50px"):
    quill_id = f"quill_display_{uuid.uuid4().hex[:8]}"

    try:
        json_content = json.dumps(content)
    except Exception:
        json_content = '{}'

    html = f"""
<div id="{quill_id}" style="border:1px solid #ccc; min-height: {height};"></div>
<script type="application/json" id="{quill_id}_data">
{json_content}
</script>

<script>
document.addEventListener("DOMContentLoaded", function() {{
    var container = document.getElementById("{quill_id}");
    var data = JSON.parse(document.getElementById("{quill_id}_data").textContent);
    var quill = new Quill(container, {{
      theme: "snow",
      readOnly: true,
      modules: {{ toolbar: false }}
    }});
    quill.setContents(data);
}});
</script>
"""
    return mark_safe(html)



from django.db import models

from quillsafe.fields import QuillField

class Article(models.Model):
    title = models.CharField(max_length=255)
    content = QuillField()

    def __str__(self):
        return str(self.title)



from django.forms.widgets import Textarea
from django.utils.safestring import mark_safe
import json

class QuillWidget(Textarea):
    def render(self, name, value, attrs=None, renderer=None):
        if isinstance(value, (dict, list)):
            value = json.dumps(value, indent=2)
        if not attrs:
            attrs = {}

        # Hide the original textarea (inline style for maximum compatibility)
        existing_style = attrs.get('style', '')
        attrs['style'] = f"{existing_style} display:none;".strip()

        textarea_html = super().render(name, value, attrs, renderer)

        textarea_id = attrs.get('id', f'id_{name}')
        quill_div_id = f'quill-editor-{name}'

        html = f'''
<div>
  <div id="{quill_div_id}" style="height: 300px; border: 1px solid #ccc; margin-top: 0.5em;"></div>
  {textarea_html}
</div>

<script>
  document.addEventListener("DOMContentLoaded", function() {{
    var quill = new Quill("#{quill_div_id}", {{
      theme: "snow"
    }});

    var textarea = document.getElementById("{textarea_id}");
    try {{
      var delta = JSON.parse(textarea.value);
      quill.setContents(delta);
    }} catch(e) {{
      quill.setText("");
    }}

    quill.on("text-change", function(delta, oldDelta, source) {{
      textarea.value = JSON.stringify(quill.getContents());
    }});
  }});
</script>
'''
        return mark_safe(html)



from django.shortcuts import render, get_object_or_404, redirect
from .models import Article
from .forms import ArticleForm

def article_create(request):
    if request.method == "POST":
        form = ArticleForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect("article_list")
    else:
        form = ArticleForm()
    return render(request, "articles/article_form.html", {"form": form})

def article_list(request):
    articles = Article.objects.all()
    return render(request, "articles/article_list.html", {"articles": articles})

def article_detail(request, pk):
    article = get_object_or_404(Article, pk=pk)
    return render(request, "articles/article_detail.html", {"article": article})



from django.urls import path
from . import views

urlpatterns = [
    path("", views.article_list, name="article_list"),
    path("new/", views.article_create, name="article_create"),
    path("<int:pk>/", views.article_detail, name="article_detail"),
]



import json
from django.db import models
from django import forms
from django.core.exceptions import ValidationError
from .widgets import QuillWidget

class QuillFormField(forms.CharField):
    widget = QuillWidget

    def to_python(self, value):
        if not value:
            return {}
        if isinstance(value, dict):
            return value
        try:
            return json.loads(value)
        except json.JSONDecodeError:
            raise ValidationError("Enter valid JSON data.")

    def validate(self, value):
        super().validate(value)
        if not isinstance(value, dict):
            raise ValidationError("Invalid format for Quill Delta JSON.")


class QuillField(models.TextField):
    description = "Field to store Quill Delta JSON data"

    def from_db_value(self, value, expression, connection):
        if value is None:
            return {}
        try:
            return json.loads(value)
        except (ValueError, TypeError):
            return {}

    def to_python(self, value):
        if isinstance(value, dict):
            return value
        if value in [None, '']:
            return {}
        try:
            return json.loads(value)
        except (ValueError, TypeError):
            raise ValidationError("Invalid JSON data")

    def get_prep_value(self, value):
        if value is None:
            return ''
        if isinstance(value, dict):
            return json.dumps(value)
        return value

    def formfield(self, **kwargs):
        defaults = {'form_class': QuillFormField}
        defaults.update(kwargs)
        return super().formfield(**defaults)
